/************************************************************************
 *
 * $Id: geos_c.h.in 2144 2008-07-31 17:01:15Z mloskot $
 *
 * C-Wrapper for GEOS library
 *
 * Copyright (C) 2005 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 * Author: Sandro Santilli <strk@refractions.net>
 *
 ***********************************************************************
 *
 * GENERAL NOTES:
 *
 *	- Remember to call initGEOS() before any use of this library's
 *	  functions, and call finishGEOS() when done.
 *
 *	- Currently you have to explicitly GEOSGeom_destroy() all
 *	  GEOSGeom objects to avoid memory leaks, and to free()
 *	  all returned char * (unless const). This might change
 *	  before first release to ensure greater API stability.
 *
 ***********************************************************************/

#ifndef GEOSTS_C_H_INCLUDED
#define GEOSTS_C_H_INCLUDED

#ifndef __cplusplus
# include <stddef.h> /* for size_t definition */
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************
 *
 * Version
 *
 ***********************************************************************/

/*
 * Following 'ifdef' hack fixes problem with generating geos_c.h on Windows,
 * when building with Visual C++ compiler.
 */
#if defined(_MSC_VER)
#include <geos/version.h>
#define GEOS_CAPI_VERSION_MAJOR 1
#define GEOS_CAPI_VERSION_MINOR 5
#define GEOS_CAPI_VERSION_PATCH 0
#define GEOS_CAPI_VERSION "3.1.0-CAPI-1.5.0"
#else
#ifndef GEOS_VERSION_MAJOR
#define GEOS_VERSION_MAJOR 3
#endif
#ifndef GEOS_VERSION_MINOR
#define GEOS_VERSION_MINOR 1
#endif
#ifndef GEOS_VERSION_PATCH
#define GEOS_VERSION_PATCH 0
#endif
#ifndef GEOS_VERSION
#define GEOS_VERSION "3.1.0"
#endif
#ifndef GEOS_JTS_PORT
#define GEOS_JTS_PORT "1.7.1"
#endif

#define GEOS_CAPI_VERSION_MAJOR 1
#define GEOS_CAPI_VERSION_MINOR 5
#define GEOS_CAPI_VERSION_PATCH 0
#define GEOS_CAPI_VERSION "3.1.0-CAPI-1.5.0"
#endif 

#define GEOS_CAPI_FIRST_INTERFACE GEOS_CAPI_VERSION_MAJOR 
#define GEOS_CAPI_LAST_INTERFACE (GEOS_CAPI_VERSION_MAJOR+GEOS_CAPI_VERSION_MINOR)
 
/************************************************************************
 *
 * (Abstract) type definitions
 *
 ***********************************************************************/

typedef void (*GEOSMessageHandler)(const char *fmt, ...);

/* When we're included by geos_c.cpp, those are #defined to the original
 * JTS definitions via preprocessor. We don't touch them to allow the
 * compiler to cross-check the declarations. However, for all "normal" 
 * C-API users, we need to define them as "opaque" struct pointers, as 
 * those clients don't have access to the original C++ headers, by design.
 */
#ifndef GEOSGeometry
typedef struct GEOSGeom_t GEOSGeometry;
typedef struct GEOSPrepGeom_t GEOSPreparedGeometry;
typedef struct GEOSCoordSeq_t GEOSCoordSequence;
#endif

/* Those are compatibility definitions for source compatibility
 * with GEOS 2.X clients relying on that type.
 */
typedef GEOSGeometry* GEOSGeom;
typedef GEOSCoordSequence* GEOSCoordSeq;

/* Supported geometry types
 * This was renamed from GEOSGeomTypeId in GEOS 2.2.X, which might
 * break compatibility, this issue is still under investigation.
 */

enum GEOSGeomTypes {
	GEOS_POINT,
	GEOS_LINESTRING,
	GEOS_LINEARRING,
	GEOS_POLYGON,
	GEOS_MULTIPOINT,
	GEOS_MULTILINESTRING,
	GEOS_MULTIPOLYGON,
	GEOS_GEOMETRYCOLLECTION
};

/* Byte oders exposed via the c api */
enum GEOSByteOrders {
	GEOS_WKB_XDR = 0, /* Big Endian */
	GEOS_WKB_NDR = 1 /* Little Endian */
};


typedef struct GEOSHandle
{
    const void *geomFactory;
    GEOSMessageHandler NOTICE_MESSAGE;
    GEOSMessageHandler ERROR_MESSAGE;
    int WKBOutputDims;
    int WKBByteOrder;
    int initialized;
} GEOSHandle_t;

/************************************************************************
 *
 * Initialization, cleanup, version
 *
 ***********************************************************************/

#if defined(_MSC_VER)
#  define GEOS_DLL     __declspec(dllexport)
#else
#  define GEOS_DLL
#endif

extern void GEOS_DLL initGEOS_TS(GEOSHandle_t *handle, 
                                 GEOSMessageHandler notice_function,
                                 GEOSMessageHandler error_function);
extern void GEOS_DLL finishGEOS_TS(GEOSHandle_t *handle);
extern const char GEOS_DLL *GEOSversion_TS();


/************************************************************************
 *
 * NOTE - These functions are DEPRECATED.  Please use the new Reader and
 * writer APIS!
 *
 ***********************************************************************/

extern GEOSGeometry GEOS_DLL *GEOSGeomFromWKT_TS(GEOSHandle_t *handle, 
                                                 const char *wkt);
extern char GEOS_DLL *GEOSGeomToWKT_TS(GEOSHandle_t *handle, 
                                       const GEOSGeometry* g);

/*
 * Specify whether output WKB should be 2d or 3d.
 * Return previously set number of dimensions.
 */
extern int GEOS_DLL GEOS_getWKBOutputDims_TS(GEOSHandle_t *handle);
extern int GEOS_DLL GEOS_setWKBOutputDims_TS(GEOSHandle_t *handle, int newDims);

/*
 * Specify whether the WKB byte order is big or little endian. 
 * The return value is the previous byte order.
 */
extern int GEOS_DLL GEOS_getWKBByteOrder_TS(GEOSHandle_t *handle);
extern int GEOS_DLL GEOS_setWKBByteOrder_TS(GEOSHandle_t *handle, 
                                            int byteOrder);

extern GEOSGeometry GEOS_DLL *GEOSGeomFromWKB_buf_TS(GEOSHandle_t *handle, 
                                                     const unsigned char *wkb,
                                                     size_t size);
extern unsigned char GEOS_DLL *GEOSGeomToWKB_buf_TS(GEOSHandle_t *handle, 
                                                    const GEOSGeometry* g, 
                                                    size_t *size);

extern GEOSGeometry GEOS_DLL *GEOSGeomFromHEX_buf_TS(GEOSHandle_t *handle, 
                                                     const unsigned char *hex,
                                                     size_t size);
extern unsigned char GEOS_DLL *GEOSGeomToHEX_buf_TS(GEOSHandle_t *handle, 
                                                    const GEOSGeometry* g,
                                                    size_t *size);

/************************************************************************
 *
 * Coordinate Sequence functions
 *
 ***********************************************************************/

/*
 * Create a Coordinate sequence with ``size'' coordinates
 * of ``dims'' dimensions.
 * Return NULL on exception.
 */
extern GEOSCoordSequence GEOS_DLL *GEOSCoordSeq_create_TS(GEOSHandle_t *handle, 
                                                          unsigned int size,
                                                          unsigned int dims);

/*
 * Clone a Coordinate Sequence.
 * Return NULL on exception.
 */
extern GEOSCoordSequence GEOS_DLL *GEOSCoordSeq_clone_TS(GEOSHandle_t *handle, 
                                                const GEOSCoordSequence* s);

/*
 * Destroy a Coordinate Sequence.
 */
extern void GEOS_DLL GEOSCoordSeq_destroy_TS(GEOSHandle_t *handle, 
                                             GEOSCoordSequence* s);

/*
 * Set ordinate values in a Coordinate Sequence.
 * Return 0 on exception.
 */
extern int GEOS_DLL GEOSCoordSeq_setX_TS(GEOSHandle_t *handle, 
                                         GEOSCoordSequence* s,
                                         unsigned int idx, double val);
extern int GEOS_DLL GEOSCoordSeq_setY_TS(GEOSHandle_t *handle, 
                                         GEOSCoordSequence* s,
                                         unsigned int idx, double val);
extern int GEOS_DLL GEOSCoordSeq_setZ_TS(GEOSHandle_t *handle, 
                                         GEOSCoordSequence* s,
                                         unsigned int idx, double val);
extern int GEOS_DLL GEOSCoordSeq_setOrdinate_TS(GEOSHandle_t *handle,
                                                GEOSCoordSequence* s,
                                                unsigned int idx,
                                                unsigned int dim, double val);

/*
 * Get ordinate values from a Coordinate Sequence.
 * Return 0 on exception.
 */
extern int GEOS_DLL GEOSCoordSeq_getX_TS(GEOSHandle_t *handle, 
                                         const GEOSCoordSequence* s,
                                         unsigned int idx, double *val);
extern int GEOS_DLL GEOSCoordSeq_getY_TS(GEOSHandle_t *handle, 
                                         const GEOSCoordSequence* s,
                                         unsigned int idx, double *val);
extern int GEOS_DLL GEOSCoordSeq_getZ_TS(GEOSHandle_t *handle, 
                                         const GEOSCoordSequence* s,
                                         unsigned int idx, double *val);
extern int GEOS_DLL GEOSCoordSeq_getOrdinate_TS(GEOSHandle_t *handle, 
                                                const GEOSCoordSequence* s,
                                                unsigned int idx,
                                                unsigned int dim, double *val);

/*
 * Get size and dimensions info from a Coordinate Sequence.
 * Return 0 on exception.
 */
extern int GEOS_DLL GEOSCoordSeq_getSize_TS(GEOSHandle_t *handle, 
                                            const GEOSCoordSequence* s,
                                            unsigned int *size);
extern int GEOS_DLL GEOSCoordSeq_getDimensions_TS(GEOSHandle_t *handle, 
                                                  const GEOSCoordSequence* s,
                                                  unsigned int *dims);


/************************************************************************
 *
 * Geometry Constructors.
 * GEOSCoordSequence* arguments will become ownership of the returned object.
 * All functions return NULL on exception.
 *
 ***********************************************************************/

extern GEOSGeometry GEOS_DLL *GEOSGeom_createPoint_TS(GEOSHandle_t *handle, 
                                                      GEOSCoordSequence* s);
extern GEOSGeometry GEOS_DLL *GEOSGeom_createLinearRing_TS(GEOSHandle_t *handle,
                                                      GEOSCoordSequence* s);
extern GEOSGeometry GEOS_DLL *GEOSGeom_createLineString_TS(GEOSHandle_t *handle,
                                                      GEOSCoordSequence* s);

/*
 * Second argument is an array of GEOSGeometry* objects.
 * The caller remains owner of the array, but pointed-to
 * objects become ownership of the returned GEOSGeometry.
 */
extern GEOSGeometry GEOS_DLL *GEOSGeom_createPolygon_TS(GEOSHandle_t *handle, 
                                                        GEOSGeometry* shell,
                                                        GEOSGeometry** holes, 
                                                        unsigned int nholes);
extern GEOSGeometry GEOS_DLL *GEOSGeom_createCollection_TS(GEOSHandle_t *handle,
                                                           int type,
                                                           GEOSGeometry* *geoms,
                                                           unsigned int ngeoms);

extern GEOSGeometry GEOS_DLL *GEOSGeom_clone_TS(GEOSHandle_t *handle, 
                                                const GEOSGeometry* g);

/************************************************************************
 *
 * Memory management
 *
 ***********************************************************************/

extern void GEOS_DLL GEOSGeom_destroy_TS(GEOSHandle_t *handle, GEOSGeometry* g);


/************************************************************************
 *
 * Topology operations - return NULL on exception.
 *
 ***********************************************************************/

extern GEOSGeometry GEOS_DLL *GEOSEnvelope_TS(GEOSHandle_t *handle, 
                                              const GEOSGeometry* g1);
extern GEOSGeometry GEOS_DLL *GEOSIntersection_TS(GEOSHandle_t *handle, 
                                                  const GEOSGeometry* g1,
                                                  const GEOSGeometry* g2);
extern GEOSGeometry GEOS_DLL *GEOSBuffer_TS(GEOSHandle_t *handle, 
                                            const GEOSGeometry* g1,
                                            double width, int quadsegs);
extern GEOSGeometry GEOS_DLL *GEOSConvexHull_TS(GEOSHandle_t *handle, 
                                                const GEOSGeometry* g1);
extern GEOSGeometry GEOS_DLL *GEOSDifference_TS(GEOSHandle_t *handle, 
                                                const GEOSGeometry* g1,
                                                const GEOSGeometry* g2);
extern GEOSGeometry GEOS_DLL *GEOSSymDifference_TS(GEOSHandle_t *handle, 
                                                   const GEOSGeometry* g1,
                                                   const GEOSGeometry* g2);
extern GEOSGeometry GEOS_DLL *GEOSBoundary_TS(GEOSHandle_t *handle, 
                                              const GEOSGeometry* g1);
extern GEOSGeometry GEOS_DLL *GEOSUnion_TS(GEOSHandle_t *handle, 
                                           const GEOSGeometry* g1,
                                           const GEOSGeometry* g2);
extern GEOSGeometry GEOS_DLL *GEOSPointOnSurface_TS(GEOSHandle_t *handle, 
                                                    const GEOSGeometry* g1);
extern GEOSGeometry GEOS_DLL *GEOSGetCentroid_TS(GEOSHandle_t *handle, 
                                                 const GEOSGeometry* g);
extern char GEOS_DLL *GEOSRelate_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1,
                                    const GEOSGeometry* g2);

/*
 * all arguments remain ownership of the caller
 * (both Geometries and pointers)
 */
extern GEOSGeometry GEOS_DLL *GEOSPolygonize_TS(GEOSHandle_t *handle, 
                              const GEOSGeometry *const geoms[],
                              unsigned int ngeoms);
extern GEOSGeometry GEOS_DLL *GEOSPolygonizer_getCutEdges_TS(
                              GEOSHandle_t *handle, 
                              const GEOSGeometry * const geoms[], 
                              unsigned int ngeoms);

extern GEOSGeometry GEOS_DLL *GEOSLineMerge_TS(GEOSHandle_t *handle, 
                                               const GEOSGeometry* g);
extern GEOSGeometry GEOS_DLL *GEOSSimplify_TS(GEOSHandle_t *handle, 
                                              const GEOSGeometry* g1,
                                              double tolerance);
extern GEOSGeometry GEOS_DLL *GEOSTopologyPreserveSimplify_TS(
                              GEOSHandle_t *handle, const GEOSGeometry* g1,
                              double tolerance);

/************************************************************************
 *
 *  Binary predicates - return 2 on exception, 1 on true, 0 on false
 *
 ***********************************************************************/

extern char GEOS_DLL GEOSRelatePattern_TS(GEOSHandle_t *handle, 
                                          const GEOSGeometry* g1, 
                                          const GEOSGeometry* g2,
                                          const char *pat);
extern char GEOS_DLL GEOSDisjoint_TS(GEOSHandle_t *handle, 
                                     const GEOSGeometry* g1,
                                     const GEOSGeometry* g2);
extern char GEOS_DLL GEOSTouches_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1,
                                    const GEOSGeometry* g2);
extern char GEOS_DLL GEOSIntersects_TS(GEOSHandle_t *handle, 
                                       const GEOSGeometry* g1,
                                       const GEOSGeometry* g2);
extern char GEOS_DLL GEOSCrosses_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1,
                                    const GEOSGeometry* g2);
extern char GEOS_DLL GEOSWithin_TS(GEOSHandle_t *handle, 
                                   const GEOSGeometry* g1,
                                   const GEOSGeometry* g2);
extern char GEOS_DLL GEOSContains_TS(GEOSHandle_t *handle, 
                                     const GEOSGeometry* g1, 
                                     const GEOSGeometry* g2);
extern char GEOS_DLL GEOSOverlaps_TS(GEOSHandle_t *handle, 
                                     const GEOSGeometry* g1,
                                     const GEOSGeometry* g2);
extern char GEOS_DLL GEOSEquals_TS(GEOSHandle_t *handle, 
                                   const GEOSGeometry* g1,
                                   const GEOSGeometry* g2);
extern char GEOS_DLL GEOSEqualsExact_TS(GEOSHandle_t *handle, 
                                        const GEOSGeometry* g1,
                                        const GEOSGeometry* g2,
                                        double tolerance);


/************************************************************************
 *
 *  Prepared Geometry Binary predicates - return 2 on exception, 1 on true, 0 on false
 *
 ***********************************************************************/

/* 
 * Argument ownership is taken by prepared geometry
 */
extern const GEOSPreparedGeometry GEOS_DLL *GEOSPrepare_TS(
                                            GEOSHandle_t *handle,
                                            const GEOSGeometry* g);

extern void GEOS_DLL GEOSPreparedGeom_destroy_TS(GEOSHandle_t *handle, 
                                                 GEOSPreparedGeometry* g);

extern char GEOS_DLL GEOSPreparedContains_TS(GEOSHandle_t *handle, 
                                             const GEOSPreparedGeometry* pg1,
                                             const GEOSGeometry* g2);
extern char GEOS_DLL GEOSPreparedContainsProperly_TS(GEOSHandle_t *handle, 
                                         const GEOSPreparedGeometry* pg1,
                                         const GEOSGeometry* g2);
extern char GEOS_DLL GEOSPreparedCovers_TS(GEOSHandle_t *handle, 
                                         const GEOSPreparedGeometry* pg1,
                                         const GEOSGeometry* g2);
extern char GEOS_DLL GEOSPreparedIntersects_TS(GEOSHandle_t *handle, 
                                         const GEOSPreparedGeometry* pg1,
                                         const GEOSGeometry* g2);


/************************************************************************
 *
 *  Unary predicate - return 2 on exception, 1 on true, 0 on false
 *
 ***********************************************************************/

extern char GEOS_DLL GEOSisEmpty_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1);
extern char GEOS_DLL GEOSisValid_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1);
extern char GEOS_DLL GEOSisSimple_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1);
extern char GEOS_DLL GEOSisRing_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1);
extern char GEOS_DLL GEOSHasZ_TS(GEOSHandle_t *handle, const GEOSGeometry* g1);


/************************************************************************
 *
 *  Geometry info
 *
 ***********************************************************************/

/* Return NULL on exception, result must be freed by caller. */
extern char GEOS_DLL *GEOSGeomType_TS(GEOSHandle_t *handle, 
                                      const GEOSGeometry* g1);

/* Return -1 on exception */
extern int GEOS_DLL GEOSGeomTypeId_TS(GEOSHandle_t *handle, 
                                      const GEOSGeometry* g1);

/* Return 0 on exception */
extern int GEOS_DLL GEOSGetSRID_TS(GEOSHandle_t *handle, 
                                   const GEOSGeometry* g1);

extern void GEOS_DLL GEOSSetSRID_TS(GEOSGeometry* g, int SRID);

/* May be called on all geometries in GEOS 3.x, returns -1 on error and 1
 * for non-multi geometries. Older GEOS versions only accept 
 * GeometryCollections or Multi* geometries here, and are likely to crash
 * when feeded simple geometries, so beware if you need compatibility with
 * old GEOS versions.
 */
extern int GEOS_DLL GEOSGetNumGeometries_TS(GEOSHandle_t *handle, 
                                            const GEOSGeometry* g1);

/*
 * Return NULL on exception, Geometry must be a Collection.
 * Returned object is a pointer to internal storage:
 * it must NOT be destroyed directly.
 */
extern const GEOSGeometry GEOS_DLL *GEOSGetGeometryN_TS(GEOSHandle_t *handle, 
                                                        const GEOSGeometry* g,
                                                        int n);

/* Return -1 on exception */
extern int GEOS_DLL GEOSNormalize_TS(GEOSHandle_t *handle, GEOSGeometry* g1);

/* Return -1 on exception */
extern int GEOS_DLL GEOSGetNumInteriorRings_TS(GEOSHandle_t *handle, 
                                               const GEOSGeometry* g1);

/*
 * Return NULL on exception, Geometry must be a Polygon.
 * Returned object is a pointer to internal storage:
 * it must NOT be destroyed directly.
 */
extern const GEOSGeometry GEOS_DLL *GEOSGetInteriorRingN_TS(
                                    GEOSHandle_t *handle, 
                                    const GEOSGeometry* g, int n);

/*
 * Return NULL on exception, Geometry must be a Polygon.
 * Returned object is a pointer to internal storage:
 * it must NOT be destroyed directly.
 */
extern const GEOSGeometry GEOS_DLL *GEOSGetExteriorRing_TS(
                                    GEOSHandle_t *handle,
                                    const GEOSGeometry* g);

/* Return -1 on exception */
extern int GEOS_DLL GEOSGetNumCoordinates_TS(GEOSHandle_t *handle, 
                                             const GEOSGeometry* g1);

/*
 * Return NULL on exception.
 * Geometry must be a LineString, LinearRing or Point.
 */
extern const GEOSCoordSequence GEOS_DLL *GEOSGeom_getCoordSeq_TS(
                                         GEOSHandle_t *handle,
                                         const GEOSGeometry* g);

/*
 * Return 0 on exception (or empty geometry)
 */
extern int GEOS_DLL GEOSGeom_getDimensions_TS(GEOSHandle_t *handle, 
                                              const GEOSGeometry* g);

/************************************************************************
 *
 *  Misc functions 
 *
 ***********************************************************************/

/* Return 0 on exception, 1 otherwise */
extern int GEOS_DLL GEOSArea_TS(GEOSHandle_t *handle, const GEOSGeometry* g1,
                                double *area);
extern int GEOS_DLL GEOSLength_TS(GEOSHandle_t *handle, const GEOSGeometry* g1, 
                                  double *length);
extern int GEOS_DLL GEOSDistance_TS(GEOSHandle_t *handle, 
                                    const GEOSGeometry* g1, 
                                    const GEOSGeometry* g2,
                                    double *dist);



/************************************************************************
 *
 * Reader and Writer APIs
 *
 ***********************************************************************/

typedef struct GEOSWKTReader_t GEOSWKTReader;
typedef struct GEOSWKTWriter_t GEOSWKTWriter;
typedef struct GEOSWKBReader_t GEOSWKBReader;
typedef struct GEOSWKBWriter_t GEOSWKBWriter;


/* WKT Reader */
extern GEOSWKTReader GEOS_DLL *GEOSWKTReader_create_TS(GEOSHandle_t *handle);
extern void GEOS_DLL GEOSWKTReader_destroy_TS(GEOSHandle_t *handle, 
                                              GEOSWKTReader* reader);
extern GEOSGeometry GEOS_DLL *GEOSWKTReader_read_TS(GEOSHandle_t *handle, 
                                                    GEOSWKTReader* reader,
                                                    const char *wkt);

/* WKT Writer */
extern GEOSWKTWriter GEOS_DLL *GEOSWKTWriter_create_TS(GEOSHandle_t *handle);
extern void GEOS_DLL GEOSWKTWriter_destroy_TS(GEOSHandle_t *handle, 
                                              GEOSWKTWriter* writer);
extern char GEOS_DLL *GEOSWKTWriter_write_TS(GEOSHandle_t *handle, 
                                             GEOSWKTWriter* reader,
                                             const GEOSGeometry* g);

/* WKB Reader */
extern GEOSWKBReader GEOS_DLL *GEOSWKBReader_create_TS(GEOSHandle_t *handle);
extern void GEOS_DLL GEOSWKBReader_destroy_TS(GEOSHandle_t *handle, 
                                              GEOSWKBReader* reader);
extern GEOSGeometry GEOS_DLL *GEOSWKBReader_read_TS(GEOSHandle_t *handle, 
                                                    GEOSWKBReader* reader,
                                                    const unsigned char *wkb,
                                                    size_t size);
extern GEOSGeometry GEOS_DLL *GEOSWKBReader_readHEX_TS(GEOSHandle_t *handle, 
                                                       GEOSWKBReader* reader,
                                                       const unsigned char *hex,
                                                       size_t size);

/* WKB Writer */
extern GEOSWKBWriter GEOS_DLL *GEOSWKBWriter_create_TS(GEOSHandle_t *handle);
extern void GEOS_DLL GEOSWKBWriter_destroy_TS(GEOSHandle_t *handle, 
                                              GEOSWKBWriter* writer);

/* The owner owns the results for these two methods! */
extern unsigned char GEOS_DLL *GEOSWKBWriter_write_TS(GEOSHandle_t *handle, 
                                                      GEOSWKBWriter* writer, 
                                                      const GEOSGeometry* g, 
                                                      size_t *size);
extern unsigned char GEOS_DLL *GEOSWKBWriter_writeHEX_TS(GEOSHandle_t *handle, 
                                                         GEOSWKBWriter* writer,
                                                         const GEOSGeometry* g,
                                                         size_t *size);

/* 
 * Specify whether output WKB should be 2d or 3d.
 * Return previously set number of dimensions.
 */
extern int GEOS_DLL GEOSWKBWriter_getOutputDimension_TS(GEOSHandle_t *handle, 
                                  const GEOSWKBWriter* writer);
extern void GEOS_DLL GEOSWKBWriter_setOutputDimension_TS(GEOSHandle_t *handle, 
                                  GEOSWKBWriter* writer, int newDimension);

/*
 * Specify whether the WKB byte order is big or little endian. 
 * The return value is the previous byte order.
 */
extern int GEOS_DLL GEOSWKBWriter_getByteOrder_TS(GEOSHandle_t *handle, 
                                                  const GEOSWKBWriter* writer);
extern void GEOS_DLL GEOSWKBWriter_setByteOrder_TS(GEOSHandle_t *handle, 
                                                   GEOSWKBWriter* writer, 
                                                   int byteOrder);

/*
 * Specify whether SRID values should be output. 
 */
extern char GEOS_DLL GEOSWKBWriter_getIncludeSRID_TS(GEOSHandle_t *handle, 
                                   const GEOSWKBWriter* writer);
extern void GEOS_DLL GEOSWKBWriter_setIncludeSRID_TS(GEOSHandle_t *handle, 
                                   GEOSWKBWriter* writer, const char writeSRID);




#ifdef __cplusplus
} // extern "C"
#endif

#endif /* #ifndef GEOSTS_C_H_INCLUDED */
